@@ -4,8 +4,6 @@ import click
 from .best_checkpoint import find_best
 from .evaluate_checkpoint import rollout
 from .experiment import experiment
-from .viskit import plot
-from .viskit import plot_export
 
 
 @click.group()
@@ -57,5 +55,3 @@ def test_module(agent_id, checkpoint):
 raylab.add_command(experiment)
 raylab.add_command(find_best)
 raylab.add_command(rollout)
-raylab.add_command(plot)
-raylab.add_command(plot_export)
