@@ -4,6 +4,7 @@ from .kl_coeff_mixin import AdaptiveKLCoeffMixin
 from .model_based import EnvFnMixin
 from .model_based import ModelSamplingMixin
 from .model_based import ModelTrainingMixin
+from .optimizer_collection import OptimizerCollection
 from .target_networks_mixin import TargetNetworksMixin
 from .torch_policy import TorchPolicy
 
@@ -14,4 +15,5 @@ __all__ = [
     "ModelTrainingMixin",
     "TargetNetworksMixin",
     "TorchPolicy",
+    "OptimizerCollection",
 ]
