@@ -5,6 +5,13 @@ import pytest
 EXPLORATION_TYPES = "ParameterNoise GaussianNoise".split(" ")
 
 
+@pytest.fixture
+def policy_cls():
+    from raylab.agents.naf import NAFTorchPolicy
+
+    return NAFTorchPolicy
+
+
 @pytest.fixture(
     params=EXPLORATION_TYPES, ids=tuple(s.split(".")[-1] for s in EXPLORATION_TYPES),
 )
@@ -12,5 +19,5 @@ def exploration(request):
     return "raylab.utils.exploration." + request.param
 
 
-def test_policy_creation(policy_and_batch_fn, exploration):
-    policy_and_batch_fn({"exploration_config": {"type": exploration}})
+def test_policy_creation(policy_cls, obs_space, action_space, exploration):
+    policy_cls(obs_space, action_space, {"exploration_config": {"type": exploration}})
